# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from pynfe.xml_sped.base import TagCaracter, TagInteiro, TagDecimal, TagDataHoraUTC
from pynfe.xml_sped.assinatura import Signature
from pynfe.xml_sped.base import ABERTURA
from .base import XMLMDFe, NAMESPACE_MDFE
from .modalFerroviario import ModalFerro
from .modalAquaviario import ModalAquav
from .modalRodoviario import ModalRodo
from .modalAereo import ModalAereo
from . import ESQUEMA_ATUAL
import os


DIRNAME = os.path.dirname(__file__)


class InfMunCarrega(XMLMDFe):
    """Informacoes dos municipios de carregamento"""
    def __init__(self):
        super(InfMunCarrega, self).__init__()
        self.cMunCarrega = TagCaracter(nome='cMunCarrega', tamanho=[7, 7], raiz='//infMunCarrega')
        self.xMunCarrega = TagCaracter(
            nome='xMunCarrega', tamanho=[2, 60], raiz='//infMunCarrega')

    @property
    def xml(self):
        xml = '<infMunCarrega>'
        xml += self.cMunCarrega.xml
        xml += self.xMunCarrega.xml
        xml += '</infMunCarrega>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.cMunCarrega.xml = arquivo
        self.xMunCarrega.xml = arquivo


class InfPercurso(XMLMDFe):
    """Informacoes do percurso do MDF-e"""
    def __init__(self):
        super(InfPercurso, self).__init__()
        self.ufPer = TagCaracter(nome='UFPer', tamanho=[2, 2], raiz='//infPercurso')

    @property
    def xml(self):
        xml = XMLMDFe.xml.fget(self)
        if self.ufPer.valor:  # Nao obrigatorio
            xml += '<infPercurso>'
            xml += self.ufPer.xml
            xml += '</infPercurso>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.ufPer.xml = arquivo


class Ide(XMLMDFe):
    """Identificacao do MDF-e"""
    def __init__(self):
        super(Ide, self).__init__()
        self.cUF = TagInteiro(nome='cUF', tamanho=[2, 2], raiz='//ide')
        self.tpAmb = TagInteiro(nome='tpAmb', tamanho=[1, 1], raiz='//ide')
        self.tpEmit = TagInteiro(nome='tpEmit', tamanho=[1, 1], raiz='//ide')
        self.tpTransp = TagInteiro(nome='tpTransp', tamanho=[1, 1], raiz='//ide', obrigatorio=False)
        self.mod = TagInteiro(nome='mod', tamanho=[2, 2], raiz='//ide', valor=58)
        self.serie = TagInteiro(nome='serie', tamanho=[1, 3], raiz='//ide')
        self.nMDF = TagInteiro(nome='nMDF', tamanho=[1, 9], raiz='//ide')
        self.cMDF = TagCaracter(nome='cMDF', tamanho=[8, 8], raiz='//ide')
        self.cDV = TagCaracter(nome='cDV', tamanho=[1, 1], raiz='//ide')
        self.modal = TagInteiro(nome='modal', tamanho=[1, 1], raiz='//ide')
        self.dhEmi = TagDataHoraUTC(nome='dhEmi', raiz='//ide')
        self.tpEmis = TagInteiro(nome='tpEmis', tamanho=[1, 1], raiz='//ide')
        self.procEmi = TagCaracter(nome='procEmi', tamanho=[1, 1], raiz='ide')
        self.verProc = TagCaracter(nome='verProc', tamanho=[1, 20], raiz='ide')
        self.ufIni = TagCaracter(nome='UFIni', tamanho=[2, 2], raiz='//ide')
        self.ufFim = TagCaracter(nome='UFFim', tamanho=[2, 2], raiz='//ide')
        self.infMunCarrega = []  # Maximo = 50
        self.infPercurso = []  # Nao obrigatorio - Maximo = 25
        self.dhIniViagem = TagDataHoraUTC(nome='dhIniViagem', raiz='//ide', obrigatorio=False)
        self.indCanalVerde = TagInteiro(
            nome='indCanalVerde', tamanho=[1, 1], raiz='//ide', obrigatorio=False)
        self.indCarregaPosterior = TagInteiro(
            nome='indCarregaPosterior', tamanho=[1, 1], raiz='ide', obrigatorio=False)

    @property
    def xml(self):
        xml = '<ide>'
        xml += self.cUF.xml
        xml += self.tpAmb.xml
        xml += self.tpEmit.xml
        xml += self.tpTransp.xml
        xml += self.mod.xml
        xml += self.serie.xml
        xml += self.nMDF.xml
        xml += self.cMDF.xml
        xml += self.cDV.xml
        xml += self.modal.xml
        xml += self.dhEmi.xml
        xml += self.tpEmis.xml
        xml += self.procEmi.xml
        xml += self.verProc.xml
        xml += self.ufIni.xml
        xml += self.ufFim.xml
        for imc in self.infMunCarrega[:50] or [InfMunCarrega()]:  # De 1 a 50 ocorrencias
            xml += imc.xml
        for ip in self.infPercurso[:25]:  # Nao obrigatorio - Maximo de 25 ocorrencias
            xml += ip.xml
        xml += self.dhIniViagem.xml
        xml += self.indCanalVerde.xml
        xml += self.indCarregaPosterior.xml
        xml += '</ide>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.cUF.xml = arquivo
            self.tpAmb.xml = arquivo
            self.tpEmit.xml = arquivo
            self.tpTransp.xml = arquivo
            self.mod.xml = arquivo
            self.serie.xml = arquivo
            self.nMDF.xml = arquivo
            self.cMDF.xml = arquivo
            self.cDV.xml = arquivo
            self.modal.xml = arquivo
            self.dhEmi.xml = arquivo
            self.tpEmis.xml = arquivo
            self.procEmi.xml = arquivo
            self.verProc.xml = arquivo
            self.ufIni.xml = arquivo
            self.ufFim.xml = arquivo
            self.infMunCarrega = self.le_grupo('//ide/infMunCarrega', InfMunCarrega)
            self.infPercurso = self.le_grupo('//ide/infPercurso', InfPercurso)
            self.dhIniViagem.xml = arquivo
            self.indCanalVerde.xml = arquivo
            self.indCarregaPosterior.xml = arquivo


class EnderEmit(XMLMDFe):
    """Endereco do emitente"""
    def __init__(self):
        super(EnderEmit, self).__init__()
        self.xLgr = TagCaracter(nome='xLgr', tamanho=[2, 60], raiz='//enderEmit')
        self.nro = TagCaracter(nome='nro', tamanho=[1, 60], raiz='//enderEmit')
        self.xCpl = TagCaracter(
            nome='xCpl', tamanho=[1, 60], raiz='//enderEmit', obrigatorio=False)
        self.xBairro = TagCaracter(nome='xBairro', tamanho=[2, 60], raiz='//enderEmit')
        self.cMun = TagCaracter(nome='cMun', tamanho=[7, 7], raiz='//enderEmit')
        self.xMun = TagCaracter(nome='xMun', tamanho=[2, 60], raiz='//enderEmit')
        self.cep = TagInteiro(
            nome='CEP', tamanho=[8, 8], raiz='//enderEmit', obrigatorio=False)
        self.uf = TagCaracter(nome='UF', tamanho=[2, 2], raiz='//enderEmit')
        self.fone = TagInteiro(
            nome='fone', tamanho=[7, 12], raiz='//enderEmit', obrigatorio=False)
        self.email = TagCaracter(
            nome='email', tamanho=[1, 60], raiz='//enderEmit', obrigatorio=False)

    @property
    def xml(self):
        xml = '<enderEmit>'
        xml += self.xLgr.xml
        xml += self.nro.xml
        xml += self.xCpl.xml
        xml += self.xBairro.xml
        xml += self.cMun.xml
        xml += self.xMun.xml
        xml += self.cep.xml
        xml += self.uf.xml
        xml += self.fone.xml
        xml += self.email.xml
        xml += '</enderEmit>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.xLgr.xml = arquivo
        self.nro.xml = arquivo
        self.xCpl.xml = arquivo
        self.xBairro.xml = arquivo
        self.cMun.xml = arquivo
        self.xMun.xml = arquivo
        self.cep.xml = arquivo
        self.uf.xml = arquivo
        self.fone.xml = arquivo
        self.email.xml = arquivo


class Emit(XMLMDFe):
    """Identificacao do emitente do MDF-e"""
    def __init__(self):
        super(Emit, self).__init__()
        self.cnpj = TagCaracter(nome='CNPJ', tamanho=[14, 14], raiz='//emit')
        self.ie = TagCaracter(nome='IE', tamanho=[2, 14], raiz='//emit')
        self.xNome = TagCaracter(nome='xNome', tamanho=[2, 60], raiz='//emit')
        self.xFant = TagCaracter(nome='xFant', tamanho=[1, 60], raiz='//emit', obrigatorio=False)
        self.enderEmit = EnderEmit()

    @property
    def xml(self):
        xml = '<emit>'
        xml += self.cnpj.xml
        xml += self.ie.xml
        xml += self.xNome.xml
        xml += self.xFant.xml
        xml += self.enderEmit.xml
        xml += '</emit>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.cnpj.xml = arquivo
        self.ie.xml = arquivo
        self.xNome.xml = arquivo
        self.xFant.xml = arquivo
        self.enderEmit.xml = arquivo


class InfResp(XMLMDFe):
    """Informacoes do responsavel pelo seguro da carga"""
    def __init__(self):
        super(InfResp, self).__init__()
        self.respSeg = TagInteiro(nome='respSeg', tamanho=[1, 1], raiz='//infResp')
        self.cnpj = TagInteiro(nome='CNPJ', tamanho=[14, 14], raiz='//infResp')
        self.cpf = TagInteiro(nome='CPF', tamanho=[11, 11], raiz='//infResp')

    @property
    def xml(self):
        xml = '<infResp>'
        xml += self.respSeg.xml
        xml += self.cnpj.xml if self.cnpj.valor else self.cpf.xml
        xml += '</infResp>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.respSeg.xml = arquivo
        self.cnpj.xml = arquivo
        self.cpf.xml = arquivo


class InfSeg(XMLMDFe):
    """Informacoes da seguradora"""
    def __init__(self):
        super(InfSeg, self).__init__()
        self.xSeg = TagCaracter(nome='xSeg', tamanho=[1, 30], raiz='//infSeg')
        self.cnpj = TagInteiro(nome='CNPJ', tamanho=[14, 14], raiz='//infSeg')

    @property
    def xml(self):
        xml = XMLMDFe.xml.fget(self)
        if all([self.xSeg.valor, self.cnpj.valor]):  # Nao obrigatorio
            xml += '<infSeg>'
            xml += self.xSeg.xml
            xml += self.cnpj.xml
            xml += '</infSeg>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.xSeg.xml = arquivo
        self.cnpj.xml = arquivo


class NAver(XMLMDFe):
    """Numero da averbacao"""
    def __init__(self):
        super(NAver, self).__init__()
        self.nAver = TagCaracter(
            nome='nAver', tamanho=[1, 40], raiz='//nAver', obrigatorio=False)

    @property
    def xml(self):
        xml = self.nAver.xml
        return xml

    @ xml.setter
    def xml(self, arquivo):
        self.nAver.xml = arquivo


class Seg(XMLMDFe):
    """Informacoes de seguranca da carga"""
    def __init__(self):
        super(Seg, self).__init__()
        self.infResp = InfResp()
        self.infSeg = InfSeg()  # Nao obrigatorio
        self.nApol = TagCaracter(
            nome='nApol', tamanho=[1, 20], raiz='//seg', obrigatorio=False)
        self.nAver = []

    @property
    def xml(self):
        xml = XMLMDFe.xml.fget(self)
        if self.nApol.valor:  # Nao obrigatorio
            xml += '<seg>'
            xml += self.infResp.xml
            xml += self.infSeg.xml
            xml += self.nApol.xml
            for na in self.nAver:
                xml += na.xml
            xml += '</seg>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.infResp.xml = arquivo
            self.infSeg.xml = arquivo
            self.nApol.xml = arquivo
            self.nAver = self.le_grupo('//seg/nAver', NAver)


class Lacres(XMLMDFe):
    """Lacres do MDF-e"""
    def __init__(self):
        super(Lacres, self).__init__()
        self.nLacre = TagCaracter(nome='nLacre', tamanho=[1, 60], raiz='//lacres')

    @property
    def xml(self):
        xml = XMLMDFe.xml.fget(self)
        if self.nLacre.valor:  # Nao obrigatorio
            xml += '<lacres>'
            xml += self.nLacre.xml
            xml += '</lacres>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.nLacre.xml = arquivo


class AutXML(XMLMDFe):
    """Autorizados para download do XML do MFD-e"""
    def __init__(self):
        super(AutXML, self).__init__()
        self.cnpj = TagCaracter(nome='CNPJ', tamanho=[14, 14], raiz='//autXML')
        self.cpf = TagCaracter(nome='CPF', tamanho=[11, 11], raiz='//autXML')

    @property
    def xml(self):
        xml = XMLMDFe.xml.fget(self)
        if all([self.cnpj.valor, self.cpf.valor]):  # Nao obrigatorio
            xml += '<autXML>'
            xml += self.cnpj.xml if self.cnpj.valor else self.cpf.xml
            xml += '</autXML>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.cnpj.xml = arquivo
        self.cpf.xml = arquivo


class InfModal(XMLMDFe):
    """Informacoes do modal: rodoviario, aereo, ferroviario ou aquaviario"""
    def __init__(self):
        super(InfModal, self).__init__()
        self.versao = TagDecimal(
            nome='infModal', tamanho=[1, 4, 1], decimais=[0, 2, 2], propriedade='versao',
            raiz='//infModal', valor='3.00')
        self.modalRodo = ModalRodo()
        self.modalAereo = ModalAereo()
        self.modalAquav = ModalAquav()
        self.modalFerro = ModalFerro()

    @property
    def xml(self):
        xml = '<infModal versaoModal="%s">' % self.versao.valor
        if self.modalRodo.veicTracao.placa.valor:
            xml += self.modalRodo.xml
        elif self.modalAereo.nac.valor:
            xml += self.modalAereo.xml
        elif self.modalAquav.irin.valor:
            xml += self.modalAquav.xml
        elif self.modalFerro.trem.xPref.valor:
            xml += self.modalFerro.xml
        xml += '</infModal>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.versao.xml = arquivo
        self.modalRodo.xml = arquivo
        self.modalAereo.xml = arquivo
        self.modalFerro.xml = arquivo
        self.modalAquav.xml = arquivo


class LacUnidTransp(XMLMDFe):
    """Lacres das unidades de transporte"""
    def __init__(self):
        super(LacUnidTransp, self).__init__()
        self.nLacre = TagCaracter(nome='nLacre', tamanho=[1, 20], raiz='//lacUnidTransp')

    @property
    def xml(self):
        xml = '<lacUnidTransp>'
        xml += self.nLacre.xml
        xml += '<lacUnidTransp>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.nLacre.xml = arquivo


class LacUnidCarga(XMLMDFe):
    """Lacres das unidades de carga"""
    def __init__(self):
        super(LacUnidCarga, self).__init__()
        self.nLacre = TagCaracter(nome='nLacre', tamanho=[1, 20], raiz='//lacUnidCarga')

    @property
    def xml(self):
        xml = '<lacUnidCarga>'
        xml += self.nLacre.xml
        xml += '</lacUnidCarga>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.nLacre.xml = arquivo


class InfUnidCarga(XMLMDFe):
    """Informacoes das unidades de carga (containers/ULD/outros)"""
    def __init__(self):
        super(InfUnidCarga, self).__init__()
        self.tpUnidCarga = TagInteiro(
            nome='tpUnidCarga', tamanho=[1, 1], raiz='//infUnidCarga')
        self.idUnidCarga = TagCaracter(nome='idUnidCarga', tamanho=[1, 20], raiz='//infUnidCarga')
        self.lacUnidCarga = []
        self.qtdRat = TagDecimal(
            nome='qtdRat', tamanho=[1, 5, 1], decimais=[0, 2, 2], raiz='//infUnidCarga')

    @property
    def xml(self):
        xml = '<infUnidCarga>'
        xml += self.tpUnidCarga.xml
        xml += self.idUnidCarga.xml
        for luc in self.lacUnidCarga:
            xml += luc.xml
        xml += self.qtdRat.xml
        xml += '</infUnidCarga>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.tpUnidCarga.xml = arquivo
            self.idUnidCarga.xml = arquivo
            self.lacUnidCarga = self.le_grupo('//infUnidCarga/lacUnidCarga', LacUnidCarga)
            self.qtdRat.xml = arquivo


class InfUnidTransp(XMLMDFe):
    """Informacoes das unidades de transporte (carreta/reboque/vagao)"""
    def __init__(self):
        super(InfUnidTransp, self).__init__()
        self.tpUnidTransp = TagInteiro(
            nome='tpUnidTransp', tamanho=[1, 1], raiz='//infUnidTransp')
        self.idUnidTransp = TagCaracter(
            nome='idUnidTransp', tamanho=[1, 20], raiz='//infUnidTransp')
        self.lacUnidTransp = []
        self.infUnidCarga = []
        self.qtdRat = TagDecimal(
            nome='qtdRat', tamanho=[1, 5, 1], decimais=[0, 2, 2], raiz='//infUnidTransp')

    @property
    def xml(self):
        xml = '<infUnidTransp>'
        xml += self.tpUnidTransp.xml
        xml += self.idUnidTransp.xml
        for lut in self.lacUnidTransp:
            xml += lut.xml
        for iuc in self.infUnidCarga:
            xml += iuc.xml
        xml += self.qtdRat.xml
        xml += '</infUnidTransp>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.tpUnidTransp.xml = arquivo
            self.idUnidTransp.xml = arquivo
            self.lacUnidTransp = self.le_grupo('//infCTe/lacUnidTransp', LacUnidTransp)
            self.infUnidCarga = self.le_grupo('//infCTe/infUnidCarga', InfUnidCarga)
            self.qtdRat.xml = arquivo


class Peri(XMLMDFe):
    """Produtos classificados pela ONU/UN como perigosos"""
    def __init__(self):
        super(Peri, self).__init__()
        self.nONU = TagCaracter(nome='nONU', tamanho=[4, 4], raiz='//peri')
        self.xNomeAE = TagCaracter(
            nome='xNomeAE', tamanho=[1, 150], raiz='//peri', obrigatorio=False)
        self.xClaRisco = TagCaracter(
            nome='xClaRisco', tamanho=[1, 40], raiz='//peri', obrigatorio=False)
        self.grEmb = TagCaracter(
            nome='grEmb', tamanho=[1, 6], raiz='//peri', obrigatorio=False)
        self.qTotProd = TagCaracter(nome='qTotProd', tamanho=[1, 20], raiz='//peri')
        self.qVolTipo = TagCaracter(
            nome='qVolTipo', tamanho=[1, 60], raiz='//peri', obrigatorio=False)

    @property
    def xml(self):
        xml = XMLMDFe.xml.fget(self)
        if self.nONU.valor:  # Nao obrigatorio
            xml += '<peri>'
            xml += self.nONU.xml
            xml += self.xNomeAE.xml
            xml += self.xClaRisco.xml
            xml += self.grEmb.xml
            xml += self.qTotProd.xml
            xml += self.qVolTipo.xml
            xml += '</peri>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.nONU.xml = arquivo
        self.xNomeAE.xml = arquivo
        self.xClaRisco.xml = arquivo
        self.grEmb.xml = arquivo
        self.qTotProd.xml = arquivo
        self.qVolTipo.xml = arquivo


class InfEntregaParcial(XMLMDFe):
    """Informacoes da entrega parcial da carga"""
    def __init__(self):
        super(InfEntregaParcial, self).__init__()
        self.qtdTotal = TagDecimal(
            nome='qtdTotal', tamanho=[1, 15, 1], decimais=[0, 4, 4], raiz='//infEntregaParcial')
        self.qtdParcial = TagDecimal(
            nome='qtdParcial', tamanho=[1, 15, 1], decimais=[0, 4, 4], raiz='//infEntregaParcial')

    @property
    def xml(self):
        xml = XMLMDFe.xml.fget(self)
        if self.qtdParcial.valor:  # Nao obrigatorio
            xml += '<infEntregaParcial>'
            xml += self.qtdTotal.xml
            xml += self.qtdParcial.xml
            xml += '</infEntregaParcial>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.qtdTotal.xml = arquivo
        self.qtdParcial.xml = arquivo


class InfCTe(XMLMDFe):
    """Conhecimento de Transporte - Quando prestador de servico de transporte"""
    def __init__(self):
        super(InfCTe, self).__init__()
        self.chCTe = TagInteiro(nome='chCTe', tamanho=[44, 44], raiz='//infCTe')
        self.segCodBarra = TagInteiro(
            nome='SegCodBarra', tamanho=[36, 36], raiz='//infCTe', obrigatorio=False)
        self.indReentrega = TagInteiro(
            nome='indReentrega', tamanho=[1, 1], raiz='//infCTe', obrigatorio=False)
        self.infUnidTransp = []  # Nao obrigatorio
        self.peri = []  # Nao obrigatorio
        self.infEntregaParcial = InfEntregaParcial()

    @property
    def xml(self):
        xml = XMLMDFe.xml.fget(self)
        if self.chCTe.valor:  # Nao obrigatorio
            xml += '<infCTe>'
            xml += self.chCTe.xml
            xml += self.segCodBarra.xml
            xml += self.indReentrega.xml
            for iut in self.infUnidTransp:
                xml += iut.xml
            for p in self.peri:
                xml += p.xml
            xml += self.infEntregaParcial.xml
            xml += '</infCTe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.chCTe.xml = arquivo
            self.segCodBarra.xml = arquivo
            self.indReentrega.xml = arquivo
            self.infUnidTransp = self.le_grupo('//infCTe/infUnidTransp', InfUnidTransp)
            self.peri = self.le_grupo('//infCTe/peri', Peri)
            self.infEntregaParcial.xml = arquivo


class InfNFe(XMLMDFe):
    """Nota Fiscal Eletronica"""
    def __init__(self):
        super(InfNFe, self).__init__()
        self.chNFe = TagCaracter(nome='chNFe', tamanho=[44, 44], raiz='//infNFe')
        self.segCodBarra = TagInteiro(
            nome='SegCodBarra', tamanho=[36, 36], raiz='//infNFe', obrigatorio=False)
        self.indReentrega = TagInteiro(
            nome='indReentrega', tamanho=[1, 1], raiz='//infNFe', obrigatorio=False)
        self.infUnidTransp = []  # Nao obrigatorio
        self.peri = []  # Nao obrigatorio

    @property
    def xml(self):
        xml = XMLMDFe.xml.fget(self)
        if self.chNFe.valor:  # Nao obrigatorio
            xml = '<infNFe>'
            xml += self.chNFe.xml
            xml += self.segCodBarra.xml
            xml += self.indReentrega.xml
            for iut in self.infUnidTransp:
                xml += iut.xml
            for p in self.peri:
                xml += p.xml
            xml += '</infNFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.chNFe.xml = arquivo
            self.segCodBarra.xml = arquivo
            self.indReentrega.xml = arquivo
            self.infUnidTransp = self.le_grupo('//infNFe/infUnidTransp', InfUnidTransp)
            self.peri = self.le_grupo('//infNFe/peri', Peri)


class InfMDFeTransp(XMLMDFe):
    """Manifesto Eletronico de Documentos Fiscais - Somente para modal aquaviario (vide MOC)"""
    def __init__(self):
        super(InfMDFeTransp, self).__init__()
        self.chMDFe = TagInteiro(nome='chMDFe', tamanho=[44, 44], raiz='//infMDFeTransp')
        self.indReentrega = TagInteiro(
            nome='indReentrega', tamanho=[1, 1], raiz='//infMDFeTransp', obrigatorio=False)
        self.infUnidTransp = []  # Nao obrigatorio
        self.peri = []  # Nao obrigatorio

    @property
    def xml(self):
        xml = XMLMDFe.xml.fget(self)
        if self.chMDFe.valor:  # Nao obrigatorio
            xml += '<infMDFeTransp>'
            xml += self.chMDFe.xml
            xml += self.indReentrega.xml
            for iut in self.infUnidTransp:
                xml += iut.xml
            for p in self.peri:
                xml += p.xml
            xml += '</infMDFeTransp>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.chMDFe.xml = arquivo
            self.indReentrega.xml = arquivo
            self.infUnidTransp = self.le_grupo('//infMDFeTransp/infUnidTransp', InfUnidTransp)
            self.peri = self.le_grupo('//infMDFeTransp/peri', Peri)


class InfMunDescarga(XMLMDFe):
    """Informacoes dos municipios de descarregamento"""
    def __init__(self):
        super(InfMunDescarga, self).__init__()
        self.cMunDescarga = TagCaracter(
            nome='cMunDescarga', tamanho=[7, 7], raiz='//infMunDescarga')
        self.xMunDescarga = TagCaracter(
            nome='xMunDescarga', tamanho=[2, 60], raiz='//infMunDescarga')
        self.infCTe = []  # Nao obrigatorio - Maximo 10000
        self.infNFe = []  # Nao obrigatorio - Maximo 10000
        self.infMDFeTransp = []  # Nao obrigatorio - Maximo 10000

    @property
    def xml(self):
        xml = '<infMunDescarga>'
        xml += self.cMunDescarga.xml
        xml += self.xMunDescarga.xml
        for cte in self.infCTe:
            xml += cte.xml
        for nfe in self.infNFe:
            xml += nfe.xml
        for mdf in self.infMDFeTransp:
            xml += mdf.xml
        xml += '</infMunDescarga>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.cMunDescarga.xml = arquivo
            self.xMunDescarga.xml = arquivo
            self.infCTe = self.le_grupo('//infMunDescarga/infCTe', InfCTe)
            self.infNFe = self.le_grupo('//infMunDescarga/infNFe', InfNFe)
            self.infMDFeTransp = self.le_grupo('//infMunDescarga/infMDFeTransp', InfMDFeTransp)


class InfDoc(XMLMDFe):
    """Informacoes dos documentos fiscais vinculados ao manifesto"""
    def __init__(self):
        super(InfDoc, self).__init__()
        self.infMunDescarga = []  # Maximo 100

    @property
    def xml(self):
        xml = '<infDoc>'
        for imd in self.infMunDescarga or [InfMunDescarga()]:  # De 1 a 100 ocorrencias
            xml += imd.xml
        xml += '</infDoc>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.infMunDescarga = self.le_grupo('//infDoc/infMunDescarga', InfMunDescarga)


class Tot(XMLMDFe):
    """Totalizadores da carga transportada e seus documentos fiscais"""
    def __init__(self):
        super(Tot, self).__init__()
        self.qCTe = TagInteiro(nome='qTCe', tamanho=[1, 6], raiz='//tot', obrigatorio=False)
        self.qNFe = TagInteiro(nome='qNFe', tamanho=[1, 6], raiz='//tot', obrigatorio=False)
        self.qMDFe = TagInteiro(nome='qMDFe', tamanho=[1, 6], raiz='//tot', obrigatorio=False)
        self.vCarga = TagDecimal(
            nome='vCarga', tamanho=[1, 15, 1], decimais=[0, 2, 2], raiz='//tot')
        self.cUnid = TagCaracter(nome='cUnid', tamanho=[2, 2], raiz='//tot')
        self.qCarga = TagDecimal(
            nome='qCarga', tamanho=[1, 15, 4], decimais=[0, 4, 4], raiz='//tot')

    @property
    def xml(self):
        xml = '<tot>'
        xml += self.qCTe.xml
        xml += self.qNFe.xml
        xml += self.qMDFe.xml
        xml += self.vCarga.xml
        xml += self.cUnid.xml
        xml += self.qCarga.xml
        xml += '</tot>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.qCTe.xml = arquivo
        self.qNFe.xml = arquivo
        self.qMDFe.xml = arquivo
        self.vCarga.xml = arquivo
        self.cUnid.xml = arquivo
        self.qCarga.xml = arquivo


class InfAdic(XMLMDFe):
    """Informacoes Adicionais"""
    def __init__(self):
        super(InfAdic, self).__init__()
        self.infAdFisco = TagCaracter(nome='infAdFisco', tamanho=[1, 2000], raiz='//infAdic')
        self.infCpl = TagCaracter(nome='infCpl', tamanho=[1, 5000], raiz='//infAdic')

    @property
    def xml(self):
        xml = XMLMDFe.xml.fget(self)
        if all([self.infAdFisco.valor, self.infCpl.valor]):  # Nao obrigatorio
            xml = '<infAdic>'
            xml += self.infAdFisco.xml
            xml += self.infCpl.xml
            xml += '</infAdic>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.infAdFisco.xml = arquivo
        self.infCpl.xml = arquivo


class InfRespTec(XMLMDFe):
    """Informacoes do responsavel tecnico pela emissao do DF-e"""
    def __init__(self):
        super(InfRespTec, self).__init__()
        self.cnpj = TagCaracter(nome='CNPJ', tamanho=[14, 14], raiz='//infRespTec', obrigatorio=False)
        self.xContato = TagCaracter(nome='xContato', tamanho=[2, 60], raiz='//infRespTec')
        self.email = TagCaracter(
            nome='email', tamanho=[6, 60], raiz='//infRespTec')
        self.fone = TagInteiro(nome='fone', tamanho=[7, 12], raiz='//infRespTec')
        self.idCSRT = TagInteiro(
            nome='idCSRT', tamanho=[3, 3], raiz='//infRespTec', obrigatorio=False)
        self.hashCSRT = TagCaracter(
            nome='hashCSRT', tamanho=[28, 28], raiz='//infRespTec', obrigatorio=False)

    @property
    def xml(self):
        xml = XMLMDFe.xml.fget(self)
        if not self.cnpj.valor:  # Nao obrigatorio
            xml = '<infRespTec>'
            xml += self.cnpj.xml
            xml += self.xContato.xml
            xml += self.email.xml
            xml += self.fone.xml
            xml += self.idCSRT.xml
            xml += self.hashCSRT.xml
            xml += '</infRespTec>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.cnpj.xml = arquivo
        self.xContato.xml = arquivo
        self.email.xml = arquivo
        self.fone.xml = arquivo
        self.idCSRT.xml = arquivo
        self.hashCSRT.xml = arquivo


class InfMDFe(XMLMDFe):
    """Informacoes do MDF-e"""
    def __init__(self):
        super(InfMDFe, self).__init__()
        self.Id = TagCaracter(nome='infMDFe', tamnho=[48, 48], propriedade='Id', raiz='/')
        self.versao = TagDecimal(
            nome='infMDFe', tamanho=[1, 4, 1], decimais=[0, 2, 2], propriedade='versao',
            raiz='/', namespace=NAMESPACE_MDFE, valor='3.00')
        self.ide = Ide()
        self.emit = Emit()
        self.infModal = InfModal()
        self.infDoc = InfDoc()
        self.seg = []  # Nao obrigatorio
        self.tot = Tot()
        self.lacres = []  # Nao obrigatorio
        self.autXML = []  # Nao obrigatorio - Maximo = 10
        self.infAdic = InfAdic()  # Nao obrigatorio
        self.infRespTec = InfRespTec()  # Nao obrigatorio

    @property
    def xml(self):
        xml = '<infMDFe Id="%s" versao="%s">' % (self.Id.valor, self.versao.valor)
        xml += self.ide.xml
        xml += self.emit.xml
        xml += self.infModal.xml
        xml += self.infDoc.xml
        for s in self.seg:
            xml += s.xml
        xml += self.tot.xml
        for lacre in self.lacres:
            xml += lacre.xml
        for a in self.autXML:
            xml += a.xml
        xml += self.infAdic.xml
        xml += self.infRespTec.xml
        xml += '</infMDFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.versao.xml = arquivo
            self.ide.xml = arquivo
            self.emit.xml = arquivo
            self.infModal.xml = arquivo
            self.infDoc.xml = arquivo
            self.seg = self.le_grupo('//infMDFe/seg', Seg)
            self.tot.xml = arquivo
            self.lacres = self.le_grupo('//infMDFe/lacres', Lacres)
            self.autXML = self.le_grupo('//infMDFe/autXML', AutXML)
            self.infAdic.xml = arquivo
            self.infRespTec.xml = arquivo


class InfMDFeSupl(XMLMDFe):
    """Informações suplementares do MDF-e"""
    def __init__(self):
        super(InfMDFeSupl, self).__init__()
        self.qrCodMDFe = TagCaracter(nome='qrCodMDFe', tamanho=[50, 500], raiz='//infMDFeSupl')

    @property
    def xml(self):
        xml = '<infMDFeSupl>'
        xml += self.qrCodMDFe.xml
        xml += '</infMDFeSupl>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.qrCodMDFe.xml = arquivo


class MDFe(XMLMDFe):
    """Manifesto Eletronico de Documentos Fiscais"""
    def __init__(self):
        super(MDFe, self).__init__()
        self.infMDFe = InfMDFe()
        self.infMDFeSupl = InfMDFeSupl()
        self.signature = Signature()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'mdfe_v3.00.xsd'

    @property
    def xml(self):
        xml = '%s<MDFe xmlns="http://www.portalfiscal.inf.br/mdfe">' % ABERTURA
        xml += self.infMDFe.xml
        xml += self.infMDFeSupl.xml
        self.signature.URI = '#' + self.infMDFe.Id.valor
        xml += self.signature.xml
        xml += '</MDFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.infMDFe.xml = arquivo
            self.infMDFeSupl.xml = arquivo
            self.signature.xml = self._le_noh('//MDFe/sig:Signature')

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
