# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from pynfe.xml_sped.base import TagDecimal, TagInteiro, TagCaracter
from . import ESQUEMA_ATUAL, NAMESPACE_MDFE
from .base import XMLMDFe
import os


DIRNAME = os.path.dirname(__file__)


class ConsMDFeNaoEnc(XMLMDFe):
    """Leiaute do pedido de consulta MDF-e nao encerrados."""
    def __init__(self):
        super(ConsMDFeNaoEnc, self).__init__()
        self.versao = TagDecimal(
            nome='consMDFeNaoEnc', tamanho=[1, 4, 1], decimais=[0, 2, 2], propriedade='versao',
            raiz='/', valor='3.00', namespace=NAMESPACE_MDFE)
        self.tpAmb = TagInteiro(nome='tpAmb', tamanho=[1, 1], raiz='//consMDFeNaoEnc')
        self.xServ = TagCaracter(
            nome='xServ', tamanho=[24, 24], raiz='//consMDFeNaoEnc',
            valor='CONSULTAR NÃO ENCERRADOS')
        self.cpf = TagCaracter(
            nome='CPF', tamanho=[11, 11], raiz='//consMDFeNaoEnc', obrigatorio=False)
        self.cnpj = TagCaracter(
            nome='CNPJ', tamanho=[14, 14], raiz='//consMDFeNaoEnc', obrigatorio=False)
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'consMDFeNaoEnc_v3.00.xsd'

    @property
    def xml(self):
        xml = '<consMDFeNaoEnc xmlns="%s" versao="%s">' % (NAMESPACE_MDFE, self.versao.valor)
        xml += self.tpAmb.xml
        xml += '<xServ>CONSULTAR NÃO ENCERRADOS</xServ>'
        xml += self.cpf.xml
        xml += self.cnpj.xml
        xml += '</consMDFeNaoEnc>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.versao.xml = arquivo
        self.tpAmb.xml = arquivo
        self.xServ.xml = arquivo
        self.cpf.xml = arquivo
        self.cnpj.xml = arquivo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
