# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from pynfe.xml_sped.base import TagCaracter, TagInteiro, TagDecimal, TagDataHora
from .base import XMLMDFe, NAMESPACE_MDFE
from . import ESQUEMA_ATUAL
import os


DIRNAME = os.path.dirname(__file__)


class Trem(XMLMDFe):
    """Informacoes da composicao do trem"""
    def __init__(self):
        super(Trem, self).__init__()
        self.xPref = TagCaracter(nome='xPref', tamanho=[1, 10], raiz='//ferrov/trem')
        self.dhTrem = TagDataHora(nome='dhTrem', raiz='//ferrov/trem', obrigatorio=False)
        self.xOri = TagCaracter(nome='xOri', tamanho=[1, 3], raiz='//ferrov/trem')
        self.xDest = TagCaracter(nome='xDest', tamanho=[1, 3], raiz='//ferrov/trem')
        self.qVag = TagInteiro(nome='qVag', tamanho=[1, 3, 1], raiz='//ferrov/trem')

    @property
    def xml(self):
        xml = '<trem>'
        xml += self.xPref.xml
        xml += self.dhTrem.xml
        xml += self.xOri.xml
        xml += self.xDest.xml
        xml += self.qVag.xml
        xml += '</trem>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.xPref.xml = arquivo
        self.dhTrem.xml = arquivo
        self.xOri.xml = arquivo
        self.xDest.xml = arquivo
        self.qVag.xml = arquivo


class Vagao(XMLMDFe):
    """Informacoes dos vagoes"""
    def __init__(self):
        super(Vagao, self).__init__()
        self.pesoBC = TagDecimal(
            nome='pesoBC', tamanho=[1, 6, 1], decimais=[3, 3, 0], raiz='//ferrov/vagao')
        self.pesoR = TagDecimal(
            nome='pesoR', tamanho=[1, 6, 1], decimais=[3, 3, 0], raiz='//ferrov/vagao')
        self.tpVag = TagCaracter(
            nome='tpVag', tamanho=[1, 1], raiz='//ferrov/vagao', obrigatorio=False)
        self.serie = TagCaracter(nome='serie', tamanho=[3, 3], raiz='//ferrov/vagao')
        self.nVag = TagInteiro(nome='nVag', tamanho=[1, 1, 1], raiz='//ferrov/vagao')
        self.nSeq = TagInteiro(
            nome='nSeq', tamanho=[1, 1, 1], raiz='//ferrov/vagao', obrigatorio=False)
        self.tu = TagDecimal(
            nome='TU', tamanho=[1, 6, 1], decimais=[3, 3, 0], raiz='//ferrov/vagao')

    @property
    def xml(self):
        xml = '<vagao>'
        xml += self.pesoBC.xml
        xml += self.pesoR.xml
        xml += self.tpVag.xml
        xml += self.serie.xml
        xml += self.nVag.xml
        xml += self.nSeq.xml
        xml += self.tu.xml
        xml += '</vagao>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.pesoBC.xml = arquivo
        self.pesoR.xml = arquivo
        self.tpVag.xml = arquivo
        self.serie.xml = arquivo
        self.nVag.xml = arquivo
        self.nSeq.xml = arquivo
        self.tu.xml = arquivo


class ModalFerro(XMLMDFe):
    """Informacoes do Modal Ferroviario"""
    def __init__(self):
        super(ModalFerro, self).__init__()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'mdfeModalFerroviario_v3.00.xsd'
        self.trem = Trem()
        self.vagao = []

    @property
    def xml(self):
        xml = '<ferrov xmlns="%s">' % NAMESPACE_MDFE
        xml += self.trem.xml
        for v in self.vagao:
            xml += v.xml
        xml += '</ferrov>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.trem.xml = arquivo
        self.vagao = self.le_grupo('//ferrov/vagao', Vagao)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
