# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from pynfe.xml_sped.base import TagCaracter, TagInteiro, TagDecimal, TagDataHoraUTC
from .MDFe import MDFe as mdfe, ABERTURA
from .base import XMLMDFe, NAMESPACE_MDFE
from . import ESQUEMA_ATUAL
import os


DIRNAME = os.path.dirname(__file__)


class InfProt(XMLMDFe):
    """Informacoes do protocolo de resposta."""
    def __init__(self):
        super(InfProt, self).__init__()
        self.Id = TagCaracter(nome='infProt', propriedade='Id', raiz='/')
        self.tpAmb = TagInteiro(nome='tpAmb', tamanho=[1, 1], raiz='//infProt')
        self.verAplic = TagCaracter(nome='verAplic', tamanho=[1, 20], raiz='//infProt')
        self.chMDFe = TagInteiro(nome='chMDFe', tamanho=[44, 44], raiz='//infProt')
        self.dhRecbto = TagDataHoraUTC(nome='dhRecbto', raiz='//infProt')
        self.nProt = TagCaracter(
            nome='nProt', tamanho=[15, 15], raiz='//infProt', obrigatorio=False)
        self.digVal = TagCaracter(
            nome='digVal', tamanho=[28, 28], raiz='//infProt', obrigatorio=False)
        self.cStat = TagCaracter(nome='cStat', tamanho=[3, 3], raiz='//infProt')
        self.xMotivo = TagCaracter(nome='xMotivo', tamanho=[1, 255], raiz='//infProt')

    @property
    def xml(self):
        xml = self.Id.xml
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.chMDFe.xml
        xml += self.dhRecbto.xml
        xml += self.nProt.xml
        xml += self.digVal.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += '</infProt>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.Id.xml = arquivo
        self.tpAmb.xml = arquivo
        self.verAplic.xml = arquivo
        self.chMDFe.xml = arquivo
        self.dhRecbto.xml = arquivo
        self.nProt.xml = arquivo
        self.digVal.xml = arquivo
        self.cStat.xml = arquivo
        self.xMotivo.xml = arquivo


class ProtMDFe(XMLMDFe):
    """Protocolo de recebimento do MDF-e."""
    def __init__(self):
        super(ProtMDFe, self).__init__()
        self.versao = TagDecimal(
            nome='protMDFe', tamanho=[1, 4, 1], decimais=[0, 2, 2], propriedade='versao',
            raiz='/', namespace=NAMESPACE_MDFE, valor='3.00')
        self.infProt = InfProt()

    @property
    def xml(self):
        xml = '<protMDFe versao="%s">' % self.versao.valor
        xml += self.infProt.xml
        xml += '</protMDFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.infProt.xml = arquivo


class ProcMDFe(XMLMDFe):
    """Compartilhamento do MDF-e."""
    def __init__(self):
        super(ProcMDFe, self).__init__()
        self.versao = TagDecimal(
            nome='procMDFe', tamanho=[1, 4, 1], decimais=[0, 2, 2], propriedade='versao',
            raiz='/', namespace=NAMESPACE_MDFE, valor='3.00')
        self.ipTransmissor = TagCaracter(
            nome='ipTransmissor', raiz='//procMDFe', obrigatorio=False)
        self.mdfe = mdfe()
        self.protMDFe = ProtMDFe()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'procMDFe_v3.00.xsd'

    @property
    def xml(self):
        xml = '%s<procMDFe xmlns="%s" versao="%s">' % (ABERTURA, NAMESPACE_MDFE, self.versao.valor)
        xml += self.ipTransmissor.xml
        xml += self.mdfe.xml.replace(ABERTURA, u'')
        xml += self.protMDFe.xml
        xml += '</procMDFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.versao.xml = arquivo
        self.ipTransmissor.xml = arquivo
        self.mdfe.xml = arquivo
        self.protMDFe.xml = arquivo


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
