# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from pynfe.xml_sped.base import TagCaracter, TagInteiro, TagDecimal, TagDataHoraUTC
from .base import XMLMDFe, NAMESPACE_MDFE
from .retConsReciMDFe import ProtMDFe
from . import ESQUEMA_ATUAL
import os


DIRNAME = os.path.dirname(__file__)


class RetMDFe(XMLMDFe):
    """Mensagem de retorno do envio de MDF-e."""
    def __init__(self):
        super(RetMDFe, self).__init__()
        self.versao = TagDecimal(
            nome='retMDFe', tamanho=[1, 4, 1], decimais=[0, 2, 2], propriedade='versao',
            raiz='/', namespace=NAMESPACE_MDFE, valor='3.00')
        self.tpAmb = TagInteiro(nome='tpAmb', tamanho=[1, 1], raiz='//retMDFe')
        self.cUF = TagInteiro(nome='cUF', tamanho=[2, 2], raiz='//retMDFe')
        self.verAplic = TagCaracter(nome='verAplic', tamanho=[1, 20], raiz='//retMDFe')
        self.cStat = TagCaracter(nome='cStat', tamanho=[3, 3], raiz='//retMDFe')
        self.xMotivo = TagCaracter(nome='xMotivo', tamanho=[1, 255], raiz='//retMDFe')
        self.protMDFe = ProtMDFe()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retMDFe_v3.00.xsd'

    @property
    def xml(self):
        xml = '<retMDFe xmlns="%s" versao="%s">' % (NAMESPACE_MDFE, self.versao.valor)
        xml += self.tpAmb.xml
        xml += self.cUF.xml
        xml += self.verAplic.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.protMDFe.xml
        xml += '</retMDFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.versao.xml = arquivo
        self.tpAmb.xml = arquivo
        self.cUF.xml = arquivo
        self.verAplic.xml = arquivo
        self.cStat.xml = arquivo
        self.xMotivo.xml = arquivo
        self.protMDFe.xml = arquivo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
