# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2017 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2017, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from pynfe.nfe.manual_600.nf_e400 import NFe as NFe_400
from .danfe import DANFENfce, DANFENfceSimples
from .processador import ProcessadorNFCe
from .retenvcancnfce import ProcEventoNFe
from geraldo.generators import PDFGenerator
from .consrecinfce import ProcNFCe
from StringIO import StringIO
import pytz

LOCAL_TZ = pytz.timezone('America/Sao_Paulo')


class NFCe(NFe_400):
    """
    Nota fiscal de consumidor eletronica
    """
    def __init__(self, certify=None, key=None, environment=2, state='', contingency=''):
        super(NFCe, self).__init__(certify, key, environment, state, contingency)
        self.process = ProcessadorNFCe(certify, key, environment, state, contingency)

    @staticmethod
    def danfe_nfce(xml_auth, xml_canc, logo='', **kwargs):
        """
        DANFE NFCe em arquivo PDF
        """
        procnfce = ProcNFCe()
        # procnfce.xml = xml_auth
        procnfce.xml = xml_auth.encode('utf-8')
        cancnfce = ProcEventoNFe()
        cancnfce.xml = xml_canc and xml_canc.encode('utf-8') or None
        danfe = DANFENfce(queryset=[procnfce], cancnfce=cancnfce, logo=logo, **kwargs)
        danfe.band_page_header = danfe.header
        danfe.band_detail = danfe.detail
        danfe.band_page_footer = danfe.footer
        pdf = StringIO()
        danfe.generate_by(PDFGenerator, filename=pdf)
        danfenfce = pdf.getvalue()
        pdf.close()

        return danfenfce

    def consultar_nota(self, chave):
        """
        Consulta da situacao atual da NF-e na Base de Dados do Portal Secretaria de Fazenda
        Estadual.
        """
        return self.process.consultar_nota(chave)

    @staticmethod
    def danfe_simples_nfce(xml):
        """
        DANFE NFC-e simplificado em arquivo PDF
        """
        procnfce = ProcNFCe()
        procnfce.xml = xml
        danfe = DANFENfceSimples(queryset=[procnfce])
        danfe.band_page_header = danfe.header
        danfe.band_detail = danfe.detail
        danfe.band_page_footer = danfe.footer
        pdf = StringIO()
        danfe.generate_by(PDFGenerator, filename=pdf)
        danfenfc = pdf.getvalue()
        pdf.close()

        return danfenfc

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
