# -*- coding: utf-8 -*-

from pynfe.xml_sped.base import *
from pynfe.xml_sped.assinatura import Signature
from . import ESQUEMA_ATUAL
import os


DIRNAME = os.path.dirname(__file__)


class InfCancEnviado(XMLNFe):
    def __init__(self):
        super(InfCancEnviado, self).__init__()
        self.Id = TagCaracter(nome='infCanc', codigo='CP03', tamanho=[46, 46], raiz='//cancNFe', propriedade='Id')
        self.tpAmb = TagInteiro(nome='tpAmb', codigo='CP05', tamanho=[ 1, 1, 1], raiz='//cancNFe/infCanc', valor=2)
        self.xServ = TagCaracter(nome='xServ', codigo='CP06', tamanho=[ 8, 8], raiz='//cancNFe/infCanc', valor='CANCELAR')
        self.chNFe = TagCaracter(nome='chNFe', codigo='CP07', tamanho=[44, 44, 44], raiz='//cancNFe/infCanc')
        self.nProt = TagCaracter(nome='nProt', codigo='CP08', tamanho=[15, 15, 15], raiz='//cancNFe/infCanc')
        self.xJust = TagCaracter(nome='xJust', codigo='CP09', tamanho=[15, 255], raiz='//cancNFe/infCanc')

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)

        self.Id.valor = 'ID' + self.chNFe.valor

        xml += self.Id.xml
        xml += self.tpAmb.xml
        xml += self.xServ.xml
        xml += self.chNFe.xml
        xml += self.nProt.xml
        xml += self.xJust.xml
        xml += '</infCanc>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.tpAmb.xml = arquivo
            self.xServ.xml = arquivo
            self.chNFe.xml = arquivo
            self.nProt.xml = arquivo
            self.xJust.xml = arquivo



class CancNFe(XMLNFe):
    def __init__(self):
        super(CancNFe, self).__init__()
        self.versao = TagDecimal(nome='cancNFe', codigo='CP01', propriedade='versao', namespace=NAMESPACE_NFE, valor='1.07', raiz='/')
        self.infCanc = InfCancEnviado()
        self.Signature = Signature()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'cancNFe_v1.07.xsd'

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.infCanc.xml

        #
        # Define a URI a ser assinada
        #
        self.Signature.URI = '#' + self.infCanc.Id.valor

        xml += self.Signature.xml
        xml += '</cancNFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.infCanc.xml = arquivo
            self.Signature.xml = self._le_noh('//cancNFe/sig:Signature')



class InfCancRecebido(XMLNFe):
    def __init__(self):
        super(InfCancRecebido, self).__init__()
        self.Id = TagCaracter(nome='infCanc', codigo='CR03', tamanho=[17, 17], raiz='//retCancNFe', propriedade='Id', obrigatorio=False)
        self.tpAmb = TagInteiro(nome='tpAmb', codigo='CR05', tamanho=[1, 1, 1], raiz='//retCancNFe/infCanc', valor=2)
        self.verAplic = TagCaracter(nome='verAplic', codigo='CR06', tamanho=[1, 20], raiz='//retCancNFe/infCanc')
        self.cStat = TagCaracter(nome='cStat', codigo='CR07', tamanho=[3, 3, 3], raiz='//retCancNFe/infCanc')
        self.xMotivo = TagCaracter(nome='xMotivo', codigo='CR08', tamanho=[1, 255], raiz='//retCancNFe/infCanc')
        self.cUF = TagInteiro(nome='cUF', codigo='CR08a', tamanho=[2, 2, 2], raiz='//retCancNFe/infCanc')
        self.chNFe = TagCaracter(nome='chNFe', codigo='CR09', tamanho=[44, 44, 44], raiz='//retcancNFe/infCanc', obrigatorio=False)
        self.dhRecbto = TagDataHora(nome='dhRecbto', codigo='CR10', raiz='//retCancNFe/infCanc', obrigatorio=False)
        self.nProt = TagCaracter(nome='nProt', codigo='CR11', tamanho=[15, 15, 15], raiz='//retCancNFe/infCanc', obrigatorio=False)

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)

        if self.Id.xml:
            xml += self.Id.xml
        else:
            xml += '<infCanc>'

        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.cUF.xml
        xml += self.chNFe.xml
        xml += self.dhRecbto.xml
        xml += self.nProt.xml
        xml += '</infCanc>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.tpAmb.xml = arquivo
            self.verAplic.xml = arquivo
            self.cStat.xml = arquivo
            self.xMotivo.xml = arquivo
            self.cUF.xml = arquivo
            self.chNFe.xml = arquivo
            self.dhRecbto.xml = arquivo
            self.nProt.xml = arquivo



class RetCancNFe(XMLNFe):
    def __init__(self):
        super(RetCancNFe, self).__init__()
        self.versao = TagDecimal(nome='retCancNFe', codigo='CR01', propriedade='versao', namespace=NAMESPACE_NFE, valor='1.07', raiz='/')
        self.infCanc = InfCancRecebido()
        self.Signature = Signature()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retCancNFe_v1.07.xsd'

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.infCanc.xml

        if len(self.Signature.URI) and (self.Signature.URI.strip() != '#'):
            xml += self.Signature.xml

        xml += '</retCancNFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.infCanc.xml = arquivo
            self.Signature.xml = self._le_noh('//retCancNFe/sig:Signature')


    def protocolo_formatado(self):
        if not self.infCanc.nProt.valor:
            return ''

        formatado = self.infCanc.nProt.valor
        formatado += ' - '
        formatado += self.infCanc.dhRecbto.formato_danfe()
        return formatado


class ProcCancNFe(XMLNFe):
    def __init__(self):
        super(ProcCancNFe, self).__init__()
        #
        # Atenção --- a tag procCancNFe tem que começar com letra minúscula, para
        # poder validar no XSD.
        #
        self.versao = TagDecimal(nome='procCancNFe', propriedade='versao', namespace=NAMESPACE_NFE, valor='1.07', raiz='/')
        self.cancNFe = CancNFe()
        self.retCancNFe = RetCancNFe()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'procCancNFe_v1.07.xsd'

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.cancNFe.xml.replace(ABERTURA, '')
        xml += self.retCancNFe.xml.replace(ABERTURA, '')
        xml += '</procCancNFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.cancNFe.xml = arquivo
            self.retCancNFe.xml = arquivo

