# -*- coding: utf-8 -*-

from pynfe.xml_sped.base import TagDecimal, TagCaracter, TagInteiro, TagDataHoraUTC
from pynfe.xml_sped.base import XMLNFe, NAMESPACE_NFE, ABERTURA
from pynfe.xml_sped.assinatura import Signature
from . import ESQUEMA_ATUAL

import os


DIRNAME = os.path.dirname(__file__)

CONDICAO_USO = 'A Carta de Correcao e disciplinada pelo paragrafo 1o-A do art. 7o do Convenio ' \
               'S/N, de 15 de dezembro de 1970 e pode ser utilizada para regularizacao de erro ' \
               'ocorrido na emissao de documento fiscal, desde que o erro nao esteja ' \
               'relacionado com: I - as variaveis que determinam o valor do imposto tais como: ' \
               'base de calculo, aliquota, diferenca de preco, quantidade, valor da operacao ou ' \
               'da prestacao; II - a correcao de dados cadastrais que implique mudanca do ' \
               'remetente ou do destinatario; III - a data de emissao ou de saida.'


class DetEvento(XMLNFe):
    def __init__(self):
        super(DetEvento, self).__init__()
        self.versao = TagDecimal(
            nome='detEvento', codigo='HP18', propriedade='versao', valor='1.00', raiz='/infEvento')
        self.descEvento = TagCaracter(
            nome='descEvento', codigo='HP19', tamanho=[5, 60, 5], raiz='//infEvento/detEvento',
            valor='Carta de Correcao')
        self.xCorrecao = TagCaracter(
            nome='xCorrecao', codigo='HP20', tamanho=[15, 1000, 15], raiz='//infEvento/detEvento')
        self.xCondUso = TagCaracter(
            nome='xCondUso', codigo='HP20a', raiz='//detEvento', valor=CONDICAO_USO)

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += self.versao.xml
        xml += self.descEvento.xml
        xml += self.xCorrecao.xml
        xml += self.xCondUso.xml
        xml += '</detEvento>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.descEvento.xml = arquivo
            self.xCorrecao.xml = arquivo
            self.xCondUso.xml = arquivo


class InfEventoEnviado(XMLNFe):
    def __init__(self):
        super(InfEventoEnviado, self).__init__()
        self.Id = TagCaracter(
            nome='infEvento', codigo='HP06', tamanho=[54, 54], raiz='//envEvento', propriedade='Id')
        self.cOrgao = TagInteiro(
            nome='cOrgao', codigo='HP08', tamanho=[2, 2, 2], raiz='//envEvento/evento/infEvento',
            valor=91)
        self.tpAmb = TagInteiro(
            nome='tpAmb', codigo='HP09', tamanho=[1, 1, 1], raiz='//envEvento/evento/infEvento',
            valor=2)
        self.CNPJ = TagCaracter(
            nome='CNPJ', codigo='HP10', tamanho=[0, 14], raiz='//envEvento/evento/infEvento')
        self.CPF = TagCaracter(
            nome='CPF', codigo='HP11', tamanho=[11, 11], raiz='//envEvento/evento/infEvento')
        self.chNFe = TagCaracter(nome='chNFe', codigo='HP12', tamanho=[44, 44, 44],
                                 raiz='//envEvento/evento/infEvento')
        self.dhEvento = TagDataHoraUTC(
            nome='dhEvento', codigo='HP13', tamanho=[0, 30], raiz='//envEvento/evento/infEvento')
        self.tpEvento = TagCaracter(
            nome='tpEvento', codigo='HP14', tamanho=[6, 6, 6], raiz='//envEvento/evento/infEvento',
            valor='110110')
        self.nSeqEvento = TagInteiro(
            nome='nSeqEvento', codigo='HP15', tamanho=[1, 2], raiz='//envEvento/evento/infEvento',
            valor=1)
        self.verEvento = TagDecimal(
            nome='verEvento', codigo='HP16', raiz='//envEvento/evento/infEvento', valor='1.00')
        self.detEvento = DetEvento()

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        self.Id.valor = 'ID' + self.tpEvento.valor + self.chNFe.valor + ("%02d" % self.nSeqEvento.valor)
        xml += self.Id.xml
        xml += self.cOrgao.xml
        xml += self.tpAmb.xml
        xml += self.CNPJ.xml
        if self.CPF.valor:
            xml += self.CPF.xml
        xml += self.chNFe.xml
        xml += self.dhEvento.xml
        xml += self.tpEvento.xml
        xml += self.nSeqEvento.xml
        xml += self.verEvento.xml
        xml += self.detEvento.xml
        xml += '</infEvento>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.cOrgao.xml = arquivo
            self.tpAmb.xml = arquivo
            self.CNPJ.xml = arquivo
            if self.CPF.valor:
                self.CPF.xml = arquivo
            self.chNFe.xml = arquivo
            self.dhEvento.xml = arquivo
            self.tpEvento.xml = arquivo
            self.nSeqEvento.xml = arquivo
            self.verEvento.xml = arquivo
            self.detEvento.xml = arquivo


class EnvEvento(XMLNFe):
    def __init__(self):
        super(EnvEvento, self).__init__()
        self.versao = TagDecimal(
            nome='envEvento', codigo='HP01', propriedade='versao', namespace=NAMESPACE_NFE,
            valor='1.00', raiz='/')
        self.idLote = TagInteiro(
            nome='idLote', codigo='HP03', tamanho=[1, 15], raiz='//envEvento', valor=1)
        self.evento_versao = TagDecimal(
            nome='evento', codigo='HP04', namespace=NAMESPACE_NFE, raiz='//envEvento', valor='1.00',
            propriedade='versao')
        self.infEvento = InfEventoEnviado()
        self.Signature = Signature()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'envCCe_v1.00.xsd'

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.idLote.xml
        xml += self.evento_versao.xml
        xml += self.infEvento.xml

        #
        # Define a URI a ser assinada
        #
        self.Signature.URI = '#' + self.infEvento.Id.valor

        xml += self.Signature.xml
        xml += '</evento>'
        xml += '</envEvento>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.infEvento.xml = arquivo
            self.Signature.xml = self._le_noh('//envEvento/sig:Signature')


class InfEventoRecebido(XMLNFe):
    def __init__(self):
        super(InfEventoRecebido, self).__init__()
        self.Id = TagCaracter(
            nome='infEvento', codigo='HR11', tamanho=[17, 17],
            raiz='//retEnvEvento/retEvento', propriedade='Id', obrigatorio=False)
        self.tpAmb = TagInteiro(
            nome='tpAmb', codigo='HR13', tamanho=[1, 1, 1],
            raiz='//retEnvEvento/retEvento/infEvento')
        self.verAplic = TagCaracter(
            nome='verAplic', codigo='HR14', tamanho=[1, 20],
            raiz='//retEnvEvento/retEvento/infEvento')
        self.cOrgao = TagInteiro(
            nome='cOrgao', codigo='HR15', tamanho=[2, 2, 2],
            raiz='//retEnvEvento/retEvento/infEvento')
        self.cStat = TagCaracter(
            nome='cStat', codigo='HR16', tamanho=[3, 3, 3],
            raiz='//retEnvEvento/retEvento/infEvento')
        self.xMotivo = TagCaracter(
            nome='xMotivo', codigo='HR17', tamanho=[1, 255],
            raiz='//retEnvEvento/retEvento/infEvento')
        self.chNFe = TagCaracter(
            nome='chNFe', codigo='HR18', tamanho=[44, 44, 44],
            raiz='//retEnvEvento/retEvento/infEvento', obrigatorio=False)
        self.tpEvento = TagCaracter(
            nome='tpEvento', codigo='HR19', tamanho=[6, 6, 6],
            raiz='//retEnvEvento/retEvento/infEvento', valor='110110', obrigatorio=False)
        self.xEvento = TagCaracter(
            nome='xEvento', codigo='HR20', tamanho=[5, 60],
            raiz='//retEnvEvento/retEvento/infEvento', obrigatorio=False)
        self.nSeqEvento = TagInteiro(
            nome='nSeqEvento', codigo='HR21', tamanho=[1, 2],
            raiz='//retEnvEvento/retEvento/infEvento', obrigatorio=False, valor=1)
        self.CNPJDest = TagCaracter(
            nome='CNPJDest', codigo='HR22', tamanho=[0, 14],
            raiz='//retEnvEvento/retEvento/infEvento', obrigatorio=False)
        self.CPFDest = TagCaracter(
            nome='CPFDest', codigo='HR23', tamanho=[11, 11],
            raiz='//retEnvEvento/retEvento/infEvento', obrigatorio=False)
        self.emailDest = TagCaracter(
            nome='emailDest', codigo='HR24', tamanho=[1, 60],
            raiz='//retEnvEvento/retEvento/infEvento', obrigatorio=False)
        self.dhRegEvento = TagCaracter(
            nome='dhRegEvento', codigo='HR25', tamanho=[0, 30],
            raiz='//retEnvEvento/retEvento/infEvento')
        self.nProt = TagCaracter(
            nome='nProt', codigo='HR26', tamanho=[15, 15, 15],
            raiz='//retEnvEvento/retEvento/infEvento', obrigatorio=False)

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)

        if self.Id.xml:
            xml += self.Id.xml
        else:
            xml += '<infEvento>'
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cOrgao.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.chNFe.xml
        xml += self.tpEvento.xml
        xml += self.xEvento.xml
        xml += self.nSeqEvento.xml
        xml += self.CNPJDest.xml
        if self.CPFDest.xml:
            xml += self.CPFDest.xml
        xml += self.emailDest.xml
        xml += self.dhRegEvento.xml
        xml += self.nProt.xml
        xml += '</infEvento>'
        xml += '</retEvento>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.tpAmb.xml = arquivo
            self.verAplic.xml = arquivo
            self.cOrgao.xml = arquivo
            self.cStat.xml = arquivo
            self.xMotivo.xml = arquivo
            self.chNFe.xml = arquivo
            self.tpEvento.xml = arquivo
            self.xEvento.xml = arquivo
            self.nSeqEvento.xml = arquivo
            self.CNPJDest.xml = arquivo
            if self.CPFDest.xml:
                self.CPFDest.xml = arquivo
            self.emailDest.xml = arquivo
            self.dhRegEvento.xml = arquivo
            self.nProt.xml = arquivo


class RetEnvEvento(XMLNFe):
    def __init__(self):
        super(RetEnvEvento, self).__init__()
        self.versao = TagDecimal(
            nome='retEnvEvento', propriedade='versao', codigo='HR01', namespace=NAMESPACE_NFE,
            valor='1.00', raiz='/')
        self.idLote = TagInteiro(
            nome='idLote', codigo='HR03', tamanho=[1, 15], raiz='//retEnvEvento', valor=1)
        self.tpAmb = TagInteiro(
            nome='tpAmb', codigo='HR04', tamanho=[1, 1, 1], raiz='//retEnvEvento', valor=2)
        self.verAplic = TagCaracter(
            nome='verAplic', codigo='HR05', tamanho=[1, 20], raiz='//retEnvEvento')
        self.cOrgao = TagInteiro(
            nome='cOrgao', codigo='HR06', tamanho=[2, 2, 2], raiz='//retEnvEvento')
        self.cStat = TagCaracter(
            nome='cStat', codigo='HR07', tamanho=[3, 3, 3], raiz='//retEnvEvento')
        self.xMotivo = TagCaracter(
            nome='xMotivo', codigo='HR08', tamanho=[1, 255], raiz='//retEnvEvento')
        self.ret_evento_versao = TagDecimal(
            nome='retEvento', codigo='HR09', raiz='//retEnvEvento', valor='1.00',
            propriedade='versao')
        self.infEvento = InfEventoRecebido()
        self.Signature = Signature()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retEnvEvento_V1.00.xsd'

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.idLote.xml
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cOrgao.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.ret_evento_versao.xml
        xml += self.infEvento.xml
        #        print 'xml evento recebido',xml

        # Define a URI a ser assinada
        #
        if len(self.Signature.URI) and (self.Signature.URI.strip() != '#'):
            xml += self.Signature.xml

        xml += '</retEnvEvento>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.idLote.xml = arquivo
            self.tpAmb.xml = arquivo
            self.verAplic.xml = arquivo
            self.cOrgao.xml = arquivo
            self.cStat.xml = arquivo
            self.xMotivo.xml = arquivo
            self.ret_evento_versao.xml = arquivo
            self.infEvento.xml = arquivo
            self.Signature.xml = self._le_noh('//retEnvEvento/sig:Signature')


class ProcEventoNFe(XMLNFe):
    def __init__(self):
        super(ProcEventoNFe, self).__init__()
        #
        # Atenção --- a tag procEventoNFe tem que começar com letra minúscula, para
        # poder validar no XSD.
        #
        self.versao = TagDecimal(
            nome='procEventoNFe', propriedade='versao', namespace=NAMESPACE_NFE, valor='1.00',
            raiz='/')
        self.envEvento = EnvEvento()
        self.retEnvEvento = RetEnvEvento()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'procCCeNFe_v1.00.xsd'

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.envEvento.evento_versao.xml.replace(ABERTURA, '')
        xml += self.envEvento.infEvento.xml
        xml += self.envEvento.Signature.xml
        xml += '</evento>'
        xml += self.retEnvEvento.ret_evento_versao.xml.replace(ABERTURA, '')
        xml += self.retEnvEvento.infEvento.xml
        xml += '</procEventoNFe>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.envEvento.evento_versao.xml = arquivo
            self.envEvento.infEvento.xml = arquivo
            self.retEnvEvento.ret_evento_versao.xml = arquivo
            self.retEnvEvento.infEvento.xml = arquivo
            self.envEvento.Signature.xml = arquivo
            self.Signature.xml = self._le_noh('//procEventoNFe/sig:Signature')

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
