# -*- coding: utf-8 -*-


import os

from pynfe.xml_sped.base import TagDecimal, TagCaracter, TagInteiro, TagDataHora, TagData
from pynfe.xml_sped.base import ABERTURA, XMLNFe, NAMESPACE_NFE
from ..manual_401 import conscad_200
from . import ESQUEMA_ATUAL


DIRNAME = os.path.dirname(__file__)


class _InfConsEnviado(XMLNFe):
    def __init__(self):
        super(_InfConsEnviado, self).__init__()
        self.xServ = TagCaracter(
            nome='xServ', codigo='GP04', tamanho=[8, 8], raiz='//ConsCad', valor='CONS-CAD')
        self.UF = TagCaracter(nome='UF', codigo='GP05', tamanho=[2, 2], raiz='//ConsCad')
        self.IE = TagCaracter(
            nome='IE', codigo='GP06', tamanho=[2, 14], raiz='//ConsCad', obrigatorio=False)
        self.CNPJ = TagCaracter(
            nome='CNPJ', codigo='GP07', tamanho=[3, 14], raiz='//ConsCad', obrigatorio=False)
        self.CPF = TagCaracter(
            nome='CPF', codigo='GP08', tamanho=[3, 11], raiz='//ConsCad', obrigatorio=False)

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += '<infCons>'
        xml += self.xServ.xml
        xml += self.UF.xml
        xml += self.IE.xml
        xml += self.CNPJ.xml
        xml += self.CPF.xml
        xml += '</infCons>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.xServ.xml = arquivo
            self.UF.xml = arquivo
            self.IE.xml = arquivo
            self.CNPJ.xml = arquivo
            self.CPF.xml = arquivo


class ConsCad(conscad_200.ConsCad):
    def __init__(self):
        super(ConsCad, self).__init__()


class Ender(conscad_200.Ender):
    def __init__(self):
        super(Ender, self).__init__()


class _InfCadRecebido(XMLNFe):
    def __init__(self):
        super(_InfCadRecebido, self).__init__()
        self.IE = TagCaracter(nome='IE', codigo='GR08', tamanho=[2, 14], raiz='//infCad')
        self.CNPJ = TagCaracter(nome='CNPJ', codigo='GR09', tamanho=[3, 14], raiz='//infCad')
        self.CPF = TagCaracter(nome='CPF', codigo='GR10', tamanho=[3, 11], raiz='//infCad')
        self.UF = TagCaracter(nome='UF', codigo='GR11', tamanho=[2, 2], raiz='//infCad')
        self.cSit = TagInteiro(nome='cSit', codigo='GR12', tamanho=[1, 1], raiz='//infCad')
        self.indCredNFe = TagCaracter(
            nome='indCredNFe', codigo='GR12a', tamanho=[1, 1], raiz='//infCad')
        self.indCredCTe = TagCaracter(
            nome='indCredCTe', codigo='GR12b', tamanho=[1, 1], raiz='//infCad')
        self.xNome = TagCaracter(
            nome='xNome', codigo='GR13', tamanho=[1, 60], raiz='//infCad', obrigatorio=False)
        self.xFant = TagCaracter(
            nome='xFant', codigo='GR13a', tamanho=[1, 60], raiz='//infCad', obrigatorio=False)
        self.xRegApur = TagCaracter(
            nome='xRegApur', codigo='GR14', tamanho=[1, 60], raiz='//infCad', obrigatorio=False)
        self.CNAE = TagInteiro(
            nome='CNAE', codigo='GR15', tamanho=[6, 7], raiz='//infCad', obrigatorio=False)
        self.dIniAtiv = TagData(nome='dIniAtiv', codigo='GR16', raiz='//infCad', obrigatorio=False)
        self.dUltSit = TagData(nome='dUltSit', codigo='GR17', raiz='//infCad', obrigatorio=False)
        self.dBaixa = TagData(nome='dBaixa', codigo='GR18', raiz='//infCad', obrigatorio=False)
        self.IEUnica = TagCaracter(
            nome='IEUnica', codigo='GR20', tamanho=[2, 14], raiz='//infCad', obrigatorio=False)
        self.IEAtual = TagCaracter(
            nome='IEAtual', codigo='GR21', tamanho=[2, 14], raiz='//infCad', obrigatorio=False)
        self.ender = Ender()

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += '<infCad>'
        xml += self.IE.xml
        xml += self.CNPJ.xml
        xml += self.CPF.xml
        xml += self.UF.xml
        xml += self.cSit.xml
        xml += self.indCredNFe.xml
        xml += self.indCredCTe.xml
        xml += self.xNome.xml
        xml += self.xFant.xml
        xml += self.xRegApur.xml
        xml += self.CNAE.xml
        xml += self.dIniAtiv.xml
        xml += self.dUltSit.xml
        xml += self.dBaixa.xml
        xml += self.IEUnica.xml
        xml += self.IEAtual.xml
        xml += self.ender.xml
        xml += '</infCad>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.IE.xml = arquivo
            self.CNPJ.xml = arquivo
            self.CPF.xml = arquivo
            self.UF.xml = arquivo
            self.cSit.xml = arquivo
            self.indCredNFe.xml = arquivo
            self.indCredCTe.xml = arquivo
            self.xNome.xml = arquivo
            self.xFant.xml = arquivo
            self.xRegApur.xml = arquivo
            self.CNAE.xml = arquivo
            self.dIniAtiv.xml = arquivo
            self.dUltSit.xml = arquivo
            self.dBaixa.xml = arquivo
            self.IEUnica.xml = arquivo
            self.IEAtual.xml = arquivo
            self.ender.xml = arquivo


class InfCad(conscad_200.InfCad):
    def __init__(self):
        super(InfCad, self).__init__()


class _InfConsRecebido(XMLNFe):
    def __init__(self):
        super(_InfConsRecebido, self).__init__()
        self.verAplic = TagCaracter(
            nome='verAplic', codigo='GR04', tamanho=[1, 20], raiz='//retConsCad/infCons')
        self.cStat = TagCaracter(
            nome='cStat', codigo='GR05', tamanho=[3, 3, 3], raiz='//retConsCad/infCons')
        self.xMotivo = TagCaracter(
            nome='xMotivo', codigo='GR06', tamanho=[1, 255], raiz='//retConsCad/infCons')
        self.UF = TagCaracter(
            nome='UF', codigo='GR06a', tamanho=[2, 2], raiz='//retConsCad/infCons')
        self.IE = TagCaracter(
            nome='IE', codigo='GR06b', tamanho=[2, 14], raiz='//retConsCad/infCons',
            obrigatorio=False)
        self.CNPJ = TagCaracter(
            nome='CNPJ', codigo='GR06c', tamanho=[3, 14], raiz='//retConsCad/infCons',
            obrigatorio=False)
        self.CPF = TagCaracter(
            nome='CPF', codigo='GR06d', tamanho=[3, 11], raiz='//retConsCad/infCons',
            obrigatorio=False)
        self.dhCons = TagDataHora(nome='dhCons', codigo='GR06e', raiz='//retConsCad/infCons')
        self.cUF = TagInteiro(
            nome='cUF', codigo='GR06f', tamanho=[2, 2, 2], raiz='//retConsCad/infCons')
        self.infCad = []

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += '<infCons>'
        xml += self.verAplic.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.UF.xml
        xml += self.IE.xml
        xml += self.CNPJ.xml
        xml += self.CPF.xml
        xml += self.dhCons.xml
        xml += self.cUF.xml
        for ic in self.infCad:
            xml += ic.xml
        xml += '</infCons>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.verAplic.xml = arquivo
            self.cStat.xml = arquivo
            self.xMotivo.xml = arquivo
            self.UF.xml = arquivo
            self.IE.xml = arquivo
            self.CNPJ.xml = arquivo
            self.CPF.xml = arquivo
            self.dhCons.xml = arquivo
            self.cUF.xml = arquivo
            self.infCad = self.le_grupo('//retConsCad/infCons/infCad', _InfCadRecebido)


class RetConsCad(conscad_200.RetConsCad):
    def __init__(self):
        super(RetConsCad, self).__init__()
        self.versao = TagDecimal(
            nome='retConsCad', codigo='GR01', propriedade='versao', namespace=NAMESPACE_NFE,
            valor='3.10', raiz='/')
        self.infCons = _InfConsRecebido()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retConsCad_v2.00.xsd'

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.infCons.xml
        xml += '</retConsCad>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.infCons.xml = arquivo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
