# -*- coding: utf-8 -*-

from pynfe.xml_sped.base import TagDecimal, TagInteiro, TagDataHoraUTC
from pynfe.xml_sped.base import XMLNFe, ABERTURA, NAMESPACE_NFE
from pynfe.nfe.manual_500 import ESQUEMA_ATUAL
from pynfe.nfe.manual_401 import envinfe_200
from .nfe_310 import NFe
import os


DIRNAME = os.path.dirname(__file__)


class EnviNFe(envinfe_200.EnviNFe):
    def __init__(self):
        super(EnviNFe, self).__init__()
        self.versao = TagDecimal(
            nome='enviNFe', codigo='AP02', propriedade='versao', namespace=NAMESPACE_NFE,
            valor='3.10', raiz='/')
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.indSinc = TagInteiro(
            nome='indSinc', codigo='AP03a', tamanho=[1, 1, 1], raiz='//enviNFe', valor=0)

        self.arquivo_esquema = 'enviNFe_v3.10.xsd'

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.idLote.xml
        xml += self.indSinc.xml
        for n in self.NFe:
            xml += n.raw_xml()
        xml += '</enviNFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.idLote.xml = arquivo
            self.indSinc.xml = arquivo
            self.NFe = self.le_grupo('//enviLote/NFe', NFe)


class InfRec(envinfe_200.InfRec):
    def __init__(self):
        super(InfRec, self).__init__()


class RetEnviNFe(envinfe_200.RetEnviNFe):
    def __init__(self):
        super(RetEnviNFe, self).__init__()
        self.versao = TagDecimal(
            nome='retEnviNFe', codigo='AR02', propriedade='versao', namespace=NAMESPACE_NFE,
            valor='3.10', raiz='/')
        self.dhRecbto = TagDataHoraUTC(nome='dhRecbto', codigo='AR06b', raiz='//retEnviNFe')
        self.infRec = InfRec()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retEnviNFe_v3.10.xsd'

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
