# -*- coding: utf-8 -*-

import os
from pynfe.xml_sped.base import TagCaracter, TagDecimal, TagDataHoraUTC
from pynfe.xml_sped.base import NAMESPACE_NFE
from ..manual_500 import ESQUEMA_ATUAL
from ..manual_401 import inutnfe_200


DIRNAME = os.path.dirname(__file__)


class InfInutEnviado(inutnfe_200.InfInutEnviado):
    def __init__(self):
        super(InfInutEnviado, self).__init__()
        self.Id = TagCaracter(
            nome='infInut', codigo='DP03', tamanho=[43, 43], raiz='//inutNFe', propriedade='Id')


class InutNFe(inutnfe_200.InutNFe):
    def __init__(self):
        super(InutNFe, self).__init__()
        self.versao = TagDecimal(
            nome='inutNFe', codigo='DP01', propriedade='versao', namespace=NAMESPACE_NFE,
            valor='3.10', raiz='/')
        self.infInut = InfInutEnviado()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'inutNFe_v3.10.xsd'


class InfInutRecebido(inutnfe_200.InfInutRecebido):
    def __init__(self):
        super(InfInutRecebido, self).__init__()
        self.dhRecbto = TagDataHoraUTC(
            nome='dhRecbto', codigo='DR16', raiz='//retInutNFe/infInut', obrigatorio=False)


class RetInutNFe(inutnfe_200.RetInutNFe):
    def __init__(self):
        super(RetInutNFe, self).__init__()
        self.versao = TagDecimal(
            nome='retInutNFe', codigo='DR01', propriedade='versao', namespace=NAMESPACE_NFE,
            valor='3.10', raiz='/')
        self.infInut = InfInutRecebido()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retInutNFe_v3.10.xsd'


class ProcInutNFe(inutnfe_200.ProcInutNFe):
    def __init__(self):
        super(ProcInutNFe, self).__init__()
        #
        # Atenção --- a tag ProcInutNFe tem que começar com letra maiúscula, para
        # poder validar no XSD. Os outros arquivos proc, procCancNFe, e procNFe
        # começam com minúscula mesmo
        #
        self.versao = TagDecimal(
            nome='ProcInutNFe', propriedade='versao', namespace=NAMESPACE_NFE, valor='3.10',
            raiz='/')
        self.inutNFe = InutNFe()
        self.retInutNFe = RetInutNFe()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'procInutNFe_v3.10.xsd'

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
