# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2017 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2017, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from .processador import ProcessadorNFe, DANFE
from ..manual_600.consrecinfe import ProtNFe, ProcNFe
from ..manual_600.nfe_400 import NFe as NFe_400
from geraldo.generators import PDFGenerator
from ..manual_600.cancnfe import RetCancNFe
from .nfe_400 import NFe as nfe_310
from datetime import datetime
from xml.dom import minidom
from io import BytesIO
import pytz

LOCAL_TZ = pytz.timezone('America/Sao_Paulo')


class NFe(nfe_310):
    """
    Nota Fiscal Eletronica
    """
    def __init__(self, certify=None, key=None, environment=2, state=u'', contingency=u''):
        super(NFe, self).__init__()
        self._certify = certify
        self._key = key
        self._environment = environment
        self._state = state
        self._contingency = contingency
        self._version = u'4.00'
        self.process = ProcessadorNFe(certify=certify, key=key, environment=environment, state=state,
                                      contingency=contingency)
        self.danfe = DANFE()

    def processar_notas(self, xml, *args):
        """
        Transmite e processa as notas fiscais eletronicas.
        """
        lista_nfe = list()
        for xml in xml if isinstance(xml, list) else [xml]:
            nfe = NFe_400()
            nfe.xml = xml
            lista_nfe.append(nfe)
        return self.process.processar_notas(lista_nfe)

    def consultar_recibo(self, lote, *args):
        """
        Realiza a consulta ao lote processado de notas fiscais.
        """
        return self.process.consultar_recibo(lote)

    def cancelar_nota(self, chave, protocolo, justificativa, *args):
        """
        Realiza o cancelamento da nota fiscal eletronica.
        """
        return self.process.cancelar_nota(chave, protocolo, justificativa)

    def inutilizar_faixa_numeracao(self, ano, cnpj, serie, inicio, fim, justificativa, *args):
        """
        Realiza a inutilizacao de numeros de notas fiscais.
        """
        self.process.estado = self._state
        self.process.certificado.cert_str = self._certify
        self.process.certificado.key_str = self._key
        self.process.ambiente = int(self._environment)
        return self.process.inutilizar_nota(ano, cnpj, serie, inicio, fim, justificativa)

    def gerar_danfe(self, xml, retcan_nfe=None, site_emitente=u'', logo=u'',
                    nome_sistema=u'SigERP - www.sigsolucoes.net.br', leiaute_logo_vertical=False):
        """
        DANFE em arquivo PDF
        """
        danfe = DANFE()
        nota = NFe_400()
        prot_nfe = ProtNFe()
        proc = ProcNFe()
        canc_nfe = RetCancNFe()
        nota.xml = xml
        xml = minidom.parseString(xml)
        xml = xml.getElementsByTagName("protNFe")[0]
        prot_nfe.xml = xml.toxml()
        proc.protNFe = prot_nfe

        # Danfe de nota fiscal cancelada
        if retcan_nfe:
            xml = minidom.parseString(retcan_nfe)
            xml = xml.getElementsByTagName('retEvento')[0]
            resposta = xml.toxml()
            canc_nfe.xml = resposta

        danfe.NFe = nota
        danfe.protNFe = prot_nfe
        danfe.retCancNFe = canc_nfe
        danfe.obs_impressao = u'DANFE gerado em %s' % self.utc_to_local(datetime.now()).strftime(
            u'%d/%m/%Y as %H:%M:%S')
        danfe.nome_sistema = nome_sistema
        danfe.site = site_emitente
        danfe.logo = logo
        danfe.leiaute_logo_vertical = leiaute_logo_vertical
        danfe.gerar_danfe()
        danfe_pdf = BytesIO()
        danfe.danfe.generate_by(PDFGenerator, filename=danfe_pdf)
        danfe.danfe = danfe_pdf.getvalue()
        danfe_pdf.close()
        return danfe.danfe

    @staticmethod
    def utc_to_local(utc_dt):
        local_dt = utc_dt.replace(tzinfo=pytz.utc).astimezone(LOCAL_TZ)
        return LOCAL_TZ.normalize(local_dt)

    def consultar_nota(self, chave):
        """
        Consulta da situacao atual da NF-e na Base de Dados do Portal Secretaria de Fazenda 
        Estadual.
        """
        return self.process.consultar_nota(chave)

    def consultar_cadastro_contribuinte(self, cpf_cnpj, inscricao_estadual=None):
        """
        Consulta dados de cadastro do contribuinte.
        """
        self.process.estado = self._state
        return self.process.consultar_cadastro_contribuinte(cpf_cnpj, inscricao_estadual)

    def efetuar_manifesto(self, cnpj, evento, chave, motivo):
        """
        Manifestacao do destinatario
        """
        return self.process.manifesto_destinatario(cnpj, evento, chave, motivo)

    def consultar_notas_destinatario(self, cnpj, nsu):
        """
        Consulta manifestacao do destinatário
        """
        return self.process.consultar_notas_destinatario(cnpj, nsu)

    def download_xml(self, cnpj, chave):
        """
        Baixa do arquivo XML da nota fiscal eletronica.
        """
        return self.process.download_xml(cnpj, chave)

    def emitir_carta_correcao(self, chave, cnpj, correcao, seq=1):
        """
        Emite carta de correcao.
        """
        return self.process.carta_correcao(chave, cnpj, correcao, seq)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
